@@ -3,18 +3,18 @@
 # for p in 5.6.2 5.8.8-nt 5.8.9d 5.10.1d 5.10.1d-nt 5.11.2d 5.11.2d-nt; do make -s clean; echo perl$p; perl$p Makefile.PL; t/testc.sh -q -O0 31; done
 # quiet c only: t/testc.sh -q -O0
 function help {
-  echo "t/testc.sh [OPTIONS] [1-29]"
-  echo " -D debugflags      for O=C or O=CC. Default: C,-DcOACMSGpu,-v resp. CC,-DoOscprSql,-v"
-  echo " -O 0|1|2|3         optimization level"
-  echo " -B static|dynamic  pass to cc_harness"
+  echo "t/testc.sh [OPTIONS] [1-$ntests]"
+  echo " -D<debugflags>     for O=C or O=CC. Default: C,-DcOACMSGpu,-v resp. CC,-DoOscprSql,-v"
+  echo " -O<0-4>            optimization level"
+  echo " -B<static|dynamic> pass to cc_harness"
   echo " -c                 continue on errors"
   echo " -k                 keep temp. files on PASS"
   echo " -E                 dump preprocessed source file with cc -E as _E.c"
-  echo " -o                 orig. no -Mblib. only for 5.6 and 5.8"
+  echo " -o                 orig. no -Mblib, use installed modules (5.6, 5.8)"
   echo " -a                 all. undo -Du. Unsilence scanning unused sub"
   echo " -q                 quiet"
   echo " -h                 help"
-  echo "Without arguments try all 29 tests. Without Option -Ox try all three optimizations."
+  echo "Without arguments try all $ntests tests. Without Option -Ox try -O0 to -O3 optimizations."
 }
 
 # use the actual perl from the Makefile (perl5.8.8, 
@@ -34,8 +34,8 @@ if [ -z $Mblib ]; then
         OCMD="$PERL $Mblib -MO=CC,-DrOsplt,"
     fi
 else
-    OCMD="$PERL $Mblib -MO=C,-DcOACMSGpu,-v,"
-    if [ $BASE = "testcc.sh" ]; then 
+    OCMD="$PERL $Mblib -MO=C,-DcoOSAHGCMpu,-v,"
+    if [ $BASE = "testcc.sh" ]; then
         OCMD="$PERL $Mblib -MO=CC,-DoOscprSql,-v,"
     fi
 fi
@@ -46,8 +46,8 @@ OCMDO4="$(echo $OCMD|sed -e s/C,-D/C,-O4,-D/)"
 CONT=
 # 5.6: rather use -B static
 #CCMD="$PERL script/cc_harness -g3"
-# rest
-CCMD="$PERL script/cc_harness -g3 -Bdynamic"
+# rest. -DALLOW_PERL_OPTIONS for -Dtlv
+CCMD="$PERL script/cc_harness -d -g3 -Bdynamic -DALLOW_PERL_OPTIONS"  
 LCMD=
 # On some perls I also had to add $archlib/DynaLoader/DynaLoader.a to libs in Config.pm
 }
@@ -138,8 +138,8 @@ function ctest {
 	    pass "./$o" "'$str' => '$res'"
             if [ -z $KEEP ]; then rm ${o}_E.c ${o}.c ${o} 2>/dev/null; fi
 	    runopt $o 1 && \
-	    runopt $o 2
-	    #runopt $o 3 && \
+	    runopt $o 2  && \
+	    runopt $o 3
 	    #runopt $o 4 && \
 	    true
 	else
@@ -149,9 +149,12 @@ function ctest {
     fi
 }
 
-ntests=31
+ntests=37
 declare -a tests[$ntests]
 declare -a result[$ntests]
+ncctests=3
+declare -a cctests[$((100+$ncctests))]
+declare -a ccresult[$((100+$ncctests))]
 tests[1]="print 'hi'"
 result[1]='hi';
 tests[2]="for (1,2,3) { print if /\d/ }"
@@ -168,22 +171,24 @@ tests[7]='@z = split /:/,"b:r:n:f:g"; print @z'
 result[7]='brnfg';
 tests[8]='sub AUTOLOAD { print 1 } &{"a"}()'
 result[8]='1';
-tests[9]='my $l = 3; $x = sub { print $l }; &$x'
+tests[9]='my $l_i = 3; $x = sub { print $l_i }; &$x'
 result[9]='3';
-tests[10]='my $i = 1; 
+tests[10]='my $i_i = 1; 
 my $foo = sub {
-  $i = shift if @_
-}; print $i; 
-print &$foo(3),$i;'
+  $i_i = shift if @_
+}; print $i_i; 
+print &$foo(3),$i_i;'
 result[10]='133';
+# index: do fbm_compile or not
 tests[11]='$x="Cannot use"; print index $x, "Can"'
 result[11]='0';
-tests[12]='my $i=6; eval "print \$i\n"'
+tests[12]='my $i_i=6; eval "print \$i_i\n"'
 result[12]='6';
 tests[13]='BEGIN { %h=(1=>2,3=>4) } print $h{3}'
 result[13]='4';
 tests[14]='open our $T,"a"; print "ok";'
 result[14]='ok';
+# __DATA__ handles still broken non-threaded 5.10
 tests[15]='print <DATA>
 __DATA__
 a
@@ -192,18 +197,18 @@ result[15]='a
 b';
 tests[16]='BEGIN{tie @a, __PACKAGE__;sub TIEARRAY {bless{}} sub FETCH{1}}; print $a[1]'
 result[16]='1';
-tests[17]='my $i=3; print 1 .. $i'
+tests[17]='my $i_ir=3; print 1 .. $i_ir'
 result[17]='123';
 # custom key sort
 tests[18]='my $h = { a=>3, b=>1 }; print sort {$h->{$a} <=> $h->{$b}} keys %$h'
 result[18]='ba';
-# fool the sort optimizer by $p, pp_sort works ok on CC
+# fool the sort optimizer by my $p, pp_sort works ok on CC
 tests[19]='print sort { my $p; $b <=> $a } 1,4,3'
 result[19]='431';
 # not repro: something like this is broken in original 5.6 (Net::DNS::ZoneFile::Fast)
 tests[20]='$a="abcd123";my $r=qr/\d/;print $a =~ $r;'
 result[20]='1';
-# broken on early alpha and 5.10
+# broken on early alpha and 5.10: run-time labels.
 tests[21]='sub skip_on_odd{next NUMBER if $_[0]% 2}NUMBER:for($i=0;$i<5;$i++){skip_on_odd($i);print $i;}'
 result[21]='024';
 # broken in original perl 5.6
@@ -222,11 +227,11 @@ result[25]="0 1 2 3`$PERL -e'print (($] < 5.007) ? q( 4 5) : q())'` 4321";
 # lvalue sub
 tests[26]='sub a:lvalue{my $a=26; ${\(bless \$a)}}sub b:lvalue{${\shift}}; print ${a(b)}';
 result[26]="26";
-# import test, AUTOLOAD goto xsub
+# xsub constants
 tests[27]='use Fcntl; print "ok" if ( &Fcntl::O_WRONLY );'
 result[27]='ok'
 # require $fname
-tests[28]='my($fname,$tmp_fh);while(!open($tmp_fh,">",($fname=q{cctest_27.} . rand(999999999999)))){$bail++;die "Failed to create a tmp file after 500 tries" if ($bail>500);}print {$tmp_fh} q{$x="ok";1;};close($tmp_fh);require $fname;unlink($fname);print $x;'
+tests[28]='my($fname,$tmp_fh);while(!open($tmp_fh,">",($fname=q{cctest28_} . rand(999999999999)))){$bail++;die "Failed to create a tmp file after 500 tries" if $bail>500;}print {$tmp_fh} q{$x="ok";1;};close($tmp_fh);require $fname;unlink($fname);print $x;'
 result[28]='ok'
 # use test
 tests[29]='use IO;print "ok"'
@@ -237,6 +242,45 @@ result[30]='456123E0'
 # AUTOLOAD w/o goto xsub
 tests[31]='package MockShell;sub AUTOLOAD{my $p=$AUTOLOAD;$p=~s/.*:://;print(join(" ",$p,@_),";");} package main; MockShell::date();MockShell::who("am","i");MockShell::ls("-l");'
 result[31]='date;who am i;ls -l;'
+# CC entertry/jmpenv_jump/leavetry
+tests[32]='eval{print "1"};eval{die 1};print "2\n";'
+result[32]='12'
+# C qr test was broken in 5.6 -- needs to load an actual file to test. See test 20.
+# used to error with Can't locate object method "save" via package "U??WVS?-" (perhaps you forgot to load "U??WVS?-"?) at /usr/lib/perl5/5.6.2/i686-linux/B/C.pm line 676.
+# fails with new constant only. still not repro
+tests[33]='package qr; 
+my $var = 1;
+my $qr_with_var = qr/^_?[^\W_0-9]\w*$var/;
+sub qr_called_in_sub { $name =~ $qr_with_var; }
+package main;
+print "ok";'
+result[33]='ok'
+# init of magic hashes. %ENV has e magic since a0714e2c perl.c  
+# (Steven Schubiger      2006-02-03 17:24:49 +0100 3967) i.e. 5.8.9 but not 5.8.8
+tests[34]='my $x=$ENV{TMPDIR};print "ok"'
+result[34]='ok'
+# method_named. fixed with 1.16
+tests[35]='package dummy;sub meth{print "ok"};package main;dummy->meth'
+result[35]='ok'
+# HV self-ref
+tests[36]='my ($rv, %hv); %hv = ( key => \$rv ); $rv = \%hv; print "ok";'
+result[36]='ok'
+# AV self-ref
+tests[37]='my ($rv, @av); @av = ( \$rv ); $rv = \@av; print "ok";'
+result[37]='ok'
+
+# from here on we test CC specifics only
+
+# CC types and arith
+tests[101]='my ($r_i,$i_i,$d_d)=(0,2,3.0); $r_i=$i_i*$i_i; $r_i*=$d_d; print $r_i;'
+result[101]='12'
+# CC cond_expr, stub, scope
+tests[102]='if ($x eq "2"){}else{print "ok"}'
+result[102]='ok'
+# CC stringify, srefgen. TODO: use B; fails
+tests[103]='require B; my $x=1e1; my $s="$x"; print ref B::svref_2object(\$s)'
+result[103]='B::PV'
+
 
 init
 
@@ -254,7 +298,7 @@ do
     OCMDO2="$(echo $OCMDO2|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO3="$(echo $OCMDO3|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
     OCMDO4="$(echo $OCMDO4|sed -e 's/-D.*,//' -e 's/,-v,/,/' -e s/-MO=/-MO=$qq/)"
-    CCMD="$PERL script/cc_harness -q -g3 -Bdynamic"
+    CCMD="$PERL script/cc_harness -q -g3 -Bdynamic -DALLOW_PERL_OPTIONS"
   fi
   if [ "$opt" = "o" ]; then Mblib=" "; init; fi
   if [ "$opt" = "c" ]; then CONT=1; fi
@@ -270,7 +314,7 @@ do
   fi
   # -B dynamic or -B static
   if [ "$opt" = "B" ]; then 
-    CCMD="$PERL script/cc_harness -g3 -B${OPTARG}"
+    CCMD="$PERL script/cc_harness -d -g3 -B${OPTARG} -DALLOW_PERL_OPTIONS"
   fi
   if [ "$opt" = "O" ]; then OPTIM="$OPTARG"; fi
   if [ "$opt" = "a" ]; then # replace -Du, by -Do
@@ -297,6 +341,11 @@ else
   for b in $(seq -f"%02.0f" $ntests); do
     ctest $b
   done
+  if [ $BASE = "testcc.sh" ]; then 
+    for b in $(seq -f"%02.0f" 101 $(($ncctests+100))); do
+      ctest $b
+    done
+  fi
 fi
 
 # 562  c:  15,25,27
